@@ -1,6 +1,6 @@
 /*
 
-Copyright 1984, 1988, 1990, 1993, 1994 by Rick Jansen, all rights reserved.
+Copyright 1984, 1988, 1990, 1993-1995 by Rick Jansen, all rights reserved.
 
   X S N O W 
         *
@@ -32,8 +32,11 @@ Copyright 1984, 1988, 1990, 1993, 1994 by Rick Jansen, all rights reserved.
  upon your request.
 
  Xsnow is also archived at ftp.x.org in the contrib/games directory.
+ Ftp to ftp.x.org and enter 'anonymous' as the login name, your
+ e-mail address as the password. Then 'cd contrib/games' and a 
+ 'get xsnow.README' for further information.
 
- If all else fails you can get a new version from me vi e-mail, as
+ If all else fails you can get a new version from me by e-mail, as
  an uuencoded and compressed tar-file. (email: rick@sara.nl)
 
  Copyright
@@ -73,10 +76,42 @@ Copyright 1984, 1988, 1990, 1993, 1994 by Rick Jansen, all rights reserved.
 
  Silicon Graphics
  ----------------
- If you run Irix 5.x with virtual desks and all, you may not see 
- any snow at all. See the README file for a solution.
+ Silicon Graphics and Irix 5.x users may not see any snow or
+ Santa at all, as long the desktop icons are visible. To
+ circumvent this problem issue this command:
+
+    /usr/lib/desktop/telldesktop quit
+
+ The icons will disappear and xsnow will work perfectly. To
+ restart the desktop just start /usr/lib/desktop/startdesktop
+ or select Desktop->Home Directory from the toolchest.
+
+ It's even possible to have both - desktop icons and xsnow
+ (and even multiple desks). You need to modify the window
+ manager's resource file 4DWm, the file ~/.desktop-
+ `hostname`/4DWm. Example:
+
+ *Global.backgroundDescription:   -execute /etc/killall -TERM
+ xsnow ; /usr/local/bin/xsnow
+ *Desk 1.backgroundDescription:   -execute /etc/killall -TERM
+ xsnow ; /usr/local/bin/xsnow
+ *Desk 2.backgroundDescription:   -execute /etc/killall -TERM
+ xsnow ; /usr/local/bin/xsnow
+
+ Restart the he toolchest and Xsnow should appear. What this
+ does is stop the currently running Xsnow and start a new one
+ when you switch to another desktop.
+
+ HP
+ --
+ HP also uses a Workspace Manager which may interfere with
+ Xsnow.  If Xsnow does not appear:  In the "Style Manager",
+ choose "Backdrop" and select "NoBackdrop".  You should now
+ be able to run Xsnow.
+
 
  Modification history
+ --------------------
  03DEC93 version 0
  04DEC93 background setting
  06DEC93 delay in main loop using select
@@ -113,12 +148,17 @@ Copyright 1984, 1988, 1990, 1993, 1994 by Rick Jansen, all rights reserved.
  20DEC94 1.38 Let Santa be a real snowplough now
  21DEC94 1.39 VMS fix for uSsleep, thanks to Ulrich Kabatek 
               <Kaba@isr.uni-stuttgart.d400.de>
-
+ 19NOV95 1.40 Memory leak fixed in CalcWindowTops, thanks to Satoshi Asami
+              <asami@cs.berkeley.edu> 
+ 19NOV95 1.40 Greatly increased performance when using solid backgrounds
+              by using XFillRectangle instead of XClearArea. Thanks to
+              Benedikt Kessler <bjk@benedikt.munich.sgi.com>
+ 20NOV95 1.40 Bug fixed with erasing snow at bot of screen
 */
 
 #define debug 0
 
-#define VERSION "Xsnow-1.39, December 21th 1994 by Rick Jansen (rick@sara.nl)"
+#define VERSION "Xsnow-1.40, November 22nd 1995 by Rick Jansen (rick@sara.nl)"
 
 #ifdef VMS
 #include <socket.h>
@@ -189,6 +229,7 @@ int NoPopuphandling = 0;
 int NoWind = 0;
 unsigned int borderWidth = 0;
 int SnowOffset = 0;
+int UseFillForClear = 0;
 
 Snow *snowflakes;
 int MaxSnowFlakeHeight = 0;  /* Biggest flake */
@@ -266,6 +307,7 @@ Pixel slcPix;
 
 /* GC's */
 GC SnowGC[SNOWFLAKEMAXTYPE+1];
+GC ClearSnowGC;
 GC SleighGCs[3];
 GC SantaGC,RudolfGC,FurGC;
 GC TreesGC;
@@ -301,89 +343,100 @@ char *av[];
     for (ax=1; ax<ac; ax++) {
 	arg = av[ax];
 	if (strcmp(arg, "-display") == 0) {
-	    display_name = av[++ax];
+	  display_name = av[++ax];
 	}
 	else if (strcmp(arg, "-sc") == 0) {
-	    snowColor = av[++ax];
+	  snowColor = av[++ax];
 	}
 	else if (strcmp(arg, "-bg") == 0) {
-	    bgColor = av[++ax];
+	  bgColor = av[++ax];
+ 
+          /* If solid colour used as bg we can use XFillArea for clear */
+          /* This results in a big performance improvement */
+          /*** Uncomment this if sure all Xservers are faster with
+               XFillRectangle than with XClearArea ***/
+          /*** UseFillForClear = 1; ***/
+        }
+        else if (strcmp(arg, "-solidbg") == 0) {
+          /* If solid colour used as bg we can use XFillArea for clear */
+          /* This results in a big performance improvement */
+          UseFillForClear = 1;
 	}
 	else if (strcmp(arg, "-tc") == 0) {
-	    trColor = av[++ax];
+	  trColor = av[++ax];
 	}
 	else if (strcmp(arg, "-slc") == 0) {
-	    slcColor = av[++ax];
+	  slcColor = av[++ax];
 	}
 	else if (strcmp(arg, "-delay") == 0) {
-	    snowDelay = strtol(av[++ax], (char **)NULL,0);
-            snowDelay = snowDelay * 1000;  /* ms to us */
+	  snowDelay = strtol(av[++ax], (char **)NULL,0);
+          snowDelay = snowDelay * 1000;  /* ms to us */
 	}
 	else if (strcmp(arg, "-snowflakes") == 0) {
-	    maxSnowflakes = strtol(av[++ax], (char **)NULL, 0);
+	  maxSnowflakes = strtol(av[++ax], (char **)NULL, 0);
 	}
         else if (strcmp(arg, "-unsmooth") == 0) {
-            SmoothWhirl = 0;
+          SmoothWhirl = 0;
         }
 	else if (strcmp(arg, "-whirl") == 0) {
-	    WhirlFactor = strtol(av[++ax], (char **)NULL, 0);
+	  WhirlFactor = strtol(av[++ax], (char **)NULL, 0);
 	}
 	else if (strcmp(arg, "-nowind") == 0) {
-	    NoWind = 1;
+	  NoWind = 1;
 	}
 	else if (strcmp(arg, "-windtimer") == 0) {
-	    WindTimer = strtol(av[++ax], (char **)NULL, 0);
+	  WindTimer = strtol(av[++ax], (char **)NULL, 0);
 	}
 	else if (strcmp(arg, "-xspeed") == 0) {
-	    MaxXStep = strtol(av[++ax], (char **)NULL, 0);
+	  MaxXStep = strtol(av[++ax], (char **)NULL, 0);
 	}
 	else if (strcmp(arg, "-yspeed") == 0) {
-	    MaxYStep = strtol(av[++ax], (char **)NULL, 0);
+	  MaxYStep = strtol(av[++ax], (char **)NULL, 0);
 	}
 	else if (strcmp(arg, "-wsnowdepth") == 0) {
-	    MaxWinSnowDepth = strtol(av[++ax], (char **)NULL, 0);
+	  MaxWinSnowDepth = strtol(av[++ax], (char **)NULL, 0);
 	}
 	else if (strcmp(arg, "-ssnowdepth") == 0) {
-	    MaxScrSnowDepth = strtol(av[++ax], (char **)NULL, 0);
+	  MaxScrSnowDepth = strtol(av[++ax], (char **)NULL, 0);
 	}
 	else if (strcmp(arg, "-offset") == 0) {
-	    SnowOffset = strtol(av[++ax], (char **)NULL, 0);
+	  SnowOffset = strtol(av[++ax], (char **)NULL, 0);
 	}
         else if (strcmp(arg, "-notrees") == 0) {
-            NoTrees = 1;     
+          NoTrees = 1;     
         }
         else if (strcmp(arg, "-nosanta") == 0) {
-            NoSanta = 1;     
+          NoSanta = 1;     
         }
         else if (strcmp(arg, "-santa") == 0) {
-	    SantaSize = strtol(av[++ax], (char **)NULL, 0);
+	  SantaSize = strtol(av[++ax], (char **)NULL, 0);
         }
         else if (strcmp(arg, "-norudolf") == 0) {
-	    Rudolf = 0;
+	  Rudolf = 0;
         }
         else if (strcmp(arg, "-santaspeed") == 0) {
-	    SantaSpeed = strtol(av[++ax], (char **)NULL, 0);
+	  SantaSpeed = strtol(av[++ax], (char **)NULL, 0);
         }
         else if (strcmp(arg, "-nokeepsnow") == 0) {
-            NoKeepSnow = 1;
-            NoKeepSWin = 1;
-            NoKeepSBot = 1;
+          NoKeepSnow = 1;
+          NoKeepSWin = 1;
+          NoKeepSBot = 1;
         }
         else if (strcmp(arg, "-nokeepsnowonwindows") == 0) {
-            NoKeepSWin = 1;
+          NoKeepSWin = 1;
         }
         else if (strcmp(arg, "-nokeepsnowonscreen") == 0) {
-            NoKeepSBot = 1;
+          NoKeepSBot = 1;
         }
         else if (strcmp(arg, "-nonopopup") == 0) {
-            NoPopuphandling = 1;
+          NoPopuphandling = 1;
         }
         else if (strcmp(arg, "-version") == 0) {
-            printf("\nThis is %s\n\n", VERSION);
-            exit(0);
+          printf("\nThis is %s\n\n", VERSION);
+          exit(0);
         }
 	else {
-	    Usage();
+	  Usage();
 	}
     }
 
@@ -510,7 +563,8 @@ char *av[];
     slcPix = AllocNamedColor(slcColor, black);
 
 
-    gc = XCreateGC(display, rootWin, 0L, &xgcv);
+    gc = XCreateGC(display, rootWin, 0, NULL);
+    XGetGCValues(display, gc, 0, &xgcv);
     XSetForeground(display, gc, blackPix);
     XSetFillStyle(display, gc, FillStippled);
 
@@ -521,6 +575,24 @@ char *av[];
       XSetWindowBackground(display, rootWin, bgPix);
       XClearWindow(display, rootWin);
       XFlush(display);
+
+      /* If -solidbg not specified notify that this may improve things! */
+      if (!UseFillForClear) {
+        printf("\nNote: when using backgrounds of one color also specifying\n");
+        printf("      -solidbg may greatly improve performance!\n");
+      }
+      else {
+        ClearSnowGC = XCreateGC(display, rootWin, 0L, &xgcv);
+        XCopyGC(display,gc,0,ClearSnowGC);
+        XSetForeground(display,ClearSnowGC, bgPix);
+      }
+    }
+    else if (UseFillForClear) {
+      printf("\n");
+      printf(
+       "Sorry, -solidbg can only be specified with a background color (-bg)");
+      printf("\n");
+      exit(1);
     }
     
     /* Fill in GCs for snowflakes */
@@ -643,21 +715,30 @@ char *av[];
                   SubR= XCreateRegion();
                     XUnionRectWithRegion(&AddRect, SubR,SubR);
                   
+                    /* Clear the snow from screen */
+                    AddY = AddY - MaxYStep-MaxSnowFlakeHeight;
+                    AddRect.height = 
+                      MaxWinSnowDepth+MaxYStep*2+MaxSnowFlakeHeight*2;
+                    if (AddY < 0) {
+                      AddRect.height = AddRect.height + AddY;
+                      AddY = 0;
+                    }
+                    AddRect.y = AddY;
+                    if ((AddY+AddRect.height) > display_height) {
+                      AddRect.height = display_height - AddY;
+                    }
+                    XUnionRectWithRegion(&AddRect, SubR,SubR);
+                    XClearArea(display, rootWin,
+                               AddX,
+                               AddY,
+                               AddRect.width,              
+                               AddRect.height,
+                               False);
+
                     /* Subtract the erased snow from the SnowCatchR */
                     XSubtractRegion(SnowCatchR,SubR, SnowCatchR);
                   XDestroyRegion(SubR);
-
-                  /* Clear the snow from screen */
-                  AddY = AddY - MaxYStep-MaxSnowFlakeHeight;
-                  AddRect.height = 
-                    MaxWinSnowDepth+MaxYStep*2+MaxSnowFlakeHeight*2;
-                  if (AddY < 0) AddY = 0;
-                  XClearArea(display, rootWin,
-                             AddX,
-                             AddY,
-                             AddRect.width,              
-                             AddRect.height,
-                             False);
+                 
 
 #if debug
 printf("exp %d %d %d %d\n",AddX,AddY,AddRect.width,AddRect.height);
@@ -750,6 +831,7 @@ sleep(4);
     exit(0);
 }
 
+
 #define USEPRT(msg) fprintf(stderr, msg)
 
 void
@@ -761,6 +843,7 @@ Usage()
     USEPRT("       -sc          <snowcolor>\n");
     USEPRT("       -tc          <tree color>\n");
     USEPRT("       -bg          <background color>\n");
+    USEPRT("       -solidbg     (Performance improvement!)\n");
     USEPRT("       -slc         <sleigh color>\n");
     USEPRT("       -snowflakes  <numsnowflakes>\n");
     USEPRT("       -delay       <delay in milliseconds>\n");
@@ -783,7 +866,7 @@ Usage()
     USEPRT("       -nokeepsnowonscreen\n");
     USEPRT("       -nonopopup\n");
     USEPRT("       -version\n\n");
-    USEPRT("Recommended: xsnow -bg LightSkyBlue3\n");
+    USEPRT("Recommended: xsnow -bg SkyBlue3\n");
     
     exit(1);
 }
@@ -1097,11 +1180,19 @@ int rx;
 
     snow = &snowflakes[rx];
    
-    XClearArea(display, rootWin, 
-               snow->intX, snow->intY,
-               snowPix[snow->whatFlake].width, 
-               snowPix[snow->whatFlake].height,
-               False);
+    if (UseFillForClear) {
+      XFillRectangle(display, rootWin, ClearSnowGC,
+                 snow->intX, snow->intY,
+                 snowPix[snow->whatFlake].width, 
+                 snowPix[snow->whatFlake].height);
+      }
+    else {
+      XClearArea(display, rootWin, 
+                 snow->intX, snow->intY,
+                 snowPix[snow->whatFlake].width, 
+                 snowPix[snow->whatFlake].height,
+                 False);
+    }
 }
 
 
@@ -1330,7 +1421,10 @@ CalcWindowTops()
        where the snow could be to the region.
     */
     for (wx=0; wx<nChildren; wx++) {
-        if (XEventsQueued(display, QueuedAlready)) return 1;
+        if (XEventsQueued(display, QueuedAlready)) {
+          XFree((char *)children); 
+          return 1;
+        }
         errorVal = 0;
         XGetWindowAttributes(display, children[wx], &wa);
         if (errorVal) continue;
